@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: pat.c 2.13 2010/06/13 11:12:12 kls Exp $
+ * $Id: pat.c 2.16 2011/04/17 13:45:25 kls Exp $
  */
 
 #include "pat.h"
@@ -13,9 +13,6 @@
 #include "libsi/section.h"
 #include "libsi/descriptor.h"
 #include "thread.h"
-#ifdef USE_TTXTSUBS
-#include "vdrttxtsubshooks.h"
-#endif /* TTXTSUBS */
 
 #define PMT_SCAN_TIMEOUT  10 // seconds
 
@@ -346,10 +343,6 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
         char DLangs[MAXDPIDS][MAXLANGCODE2] = { "" };
         char SLangs[MAXSPIDS][MAXLANGCODE2] = { "" };
         int Tpid = 0;
-#ifdef USE_TTXTSUBS
-        tTeletextSubtitlePage TeletextSubtitlePages[MAXTXTPAGES];
-        int NumTPages = 0;
-#endif /* TTXTSUBS */
         int NumApids = 0;
         int NumDpids = 0;
         int NumSpids = 0;
@@ -441,27 +434,8 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                                     NumSpids++;
                                     }
                                  break;
-#ifdef USE_TTXTSUBS
-                            case SI::TeletextDescriptorTag: {
-#else
                             case SI::TeletextDescriptorTag:
-#endif /* TTXTSUBS */
                                  Tpid = esPid;
-#ifdef USE_TTXTSUBS
-                                 SI::TeletextDescriptor *sd = (SI::TeletextDescriptor *)d;
-                                 SI::TeletextDescriptor::Teletext ttxt;
-                                 for (SI::Loop::Iterator it; sd->teletextLoop.getNext(ttxt, it); ) {
-                                     bool isSubtitlePage = (ttxt.getTeletextType() == 0x02) || (ttxt.getTeletextType() == 0x05);
-                                     if ((NumTPages < MAXTXTPAGES) && ttxt.languageCode[0] && isSubtitlePage) {
-                                        strn0cpy(TeletextSubtitlePages[NumTPages].ttxtLanguage, I18nNormalizeLanguageCode(ttxt.languageCode), MAXLANGCODE1);
-                                        TeletextSubtitlePages[NumTPages].ttxtPage = ttxt.getTeletextPageNumber();
-                                        TeletextSubtitlePages[NumTPages].ttxtMagazine = ttxt.getTeletextMagazineNumber();
-                                        TeletextSubtitlePages[NumTPages].ttxtType = ttxt.getTeletextType();
-                                        NumTPages++;
-                                        }
-                                     }
-                                 }
-#endif /* TTXTSUBS */
                                  break;
                             case SI::ISO639LanguageDescriptorTag: {
                                  SI::ISO639LanguageDescriptor *ld = (SI::ISO639LanguageDescriptor *)d;
@@ -482,21 +456,66 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                          }
                       }
                       break;
-              case 0x81: // STREAMTYPE_USER_PRIVATE
-                      if (Channel->IsAtsc()) { // ATSC AC-3
-                         char lang[MAXLANGCODE1] = { 0 };
-                         SI::Descriptor *d;
-                         for (SI::Loop::Iterator it; (d = stream.streamDescriptors.getNext(it)); ) {
-                             switch (d->getDescriptorTag()) {
-                               case SI::ISO639LanguageDescriptorTag: {
-                                    SI::ISO639LanguageDescriptor *ld = (SI::ISO639LanguageDescriptor *)d;
-                                    strn0cpy(lang, I18nNormalizeLanguageCode(ld->languageCode), MAXLANGCODE1);
-                                    }
-                                    break;
-                               default: ;
-                               }
-                             delete d;
-                             }
+              case 0x80: // STREAMTYPE_USER_PRIVATE - DigiCipher II VIDEO (ANSI/SCTE 57)
+                      Vpid = esPid;
+                      Ppid = pmt.getPCRPid();
+                      Vtype = 0x02; // compression based upon MPEG-2
+                      ProcessCaDescriptors = true;
+                      break;
+              case 0x81: // STREAMTYPE_USER_PRIVATE - ATSC A/53 AUDIO (ANSI/SCTE 57)
+                      {
+                      char lang[MAXLANGCODE1] = { 0 };
+                      SI::Descriptor *d;
+                      for (SI::Loop::Iterator it; (d = stream.streamDescriptors.getNext(it)); ) {
+                          switch (d->getDescriptorTag()) {
+                            case SI::ISO639LanguageDescriptorTag: {
+                                 SI::ISO639LanguageDescriptor *ld = (SI::ISO639LanguageDescriptor *)d;
+                                 strn0cpy(lang, I18nNormalizeLanguageCode(ld->languageCode), MAXLANGCODE1);
+                                 }
+                                 break;
+                            default: ;
+                            }
+                         delete d;
+                         }
+                      if (NumDpids < MAXDPIDS) {
+                         Dpids[NumDpids] = esPid;
+                         Dtypes[NumDpids] = SI::AC3DescriptorTag;
+                         strn0cpy(DLangs[NumDpids], lang, MAXLANGCODE1);
+                         NumDpids++;
+                         }
+                      ProcessCaDescriptors = true;
+                      }
+                      break;
+              case 0x82 ... 0xFF: // STREAMTYPE_USER_PRIVATE
+                      {
+                      char lang[MAXLANGCODE1] = { 0 };
+                      bool IsAc3 = false;
+                      SI::Descriptor *d;
+                      for (SI::Loop::Iterator it; (d = stream.streamDescriptors.getNext(it)); ) {
+                          switch (d->getDescriptorTag()) {
+                            case SI::RegistrationDescriptorTag: {
+                                 SI::RegistrationDescriptor *rd = (SI::RegistrationDescriptor *)d;
+                                 // http://www.smpte-ra.org/mpegreg/mpegreg.html
+                                 switch (rd->getFormatIdentifier()) {
+                                   case 0x41432D33: // 'AC-3'
+                                        IsAc3 = true;
+                                        break;
+                                   default:
+                                        //printf("Format identifier: 0x%08X (pid: %d)\n", rd->getFormatIdentifier(), esPid);
+                                        break;
+                                   }
+                                 }
+                                 break;
+                            case SI::ISO639LanguageDescriptorTag: {
+                                 SI::ISO639LanguageDescriptor *ld = (SI::ISO639LanguageDescriptor *)d;
+                                 strn0cpy(lang, I18nNormalizeLanguageCode(ld->languageCode), MAXLANGCODE1);
+                                 }
+                                 break;
+                            default: ;
+                            }
+                         delete d;
+                         }
+                      if (IsAc3) {
                          if (NumDpids < MAXDPIDS) {
                             Dpids[NumDpids] = esPid;
                             Dtypes[NumDpids] = SI::AC3DescriptorTag;
@@ -505,6 +524,7 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
                             }
                          ProcessCaDescriptors = true;
                          }
+                      }
                       break;
               default: ;//printf("PID: %5d %5d %2d %3d %3d\n", pmt.getServiceId(), stream.getPid(), stream.getStreamType(), pmt.getVersionNumber(), Channel->Number());
               }
@@ -517,17 +537,7 @@ void cPatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length
             }
         if (Setup.UpdateChannels >= 2) {
            Channel->SetPids(Vpid, Ppid, Vtype, Apids, Atypes, ALangs, Dpids, Dtypes, DLangs, Spids, SLangs, Tpid);
-#ifdef USE_TTXTSUBS
-           if (NumTPages < MAXTXTPAGES) {
-	      int manualPageNumber = cVDRTtxtsubsHookListener::Hook()->ManualPageNumber(Channel);
-	      if (manualPageNumber)
-                 TeletextSubtitlePages[NumTPages++] = tTeletextSubtitlePage(manualPageNumber);
-              }
-           Channel->SetTeletextSubtitlePages(TeletextSubtitlePages, NumTPages);
-	   Channel->SetCaIds(CaDescriptors->CaIds());
-#else
            Channel->SetCaIds(CaDescriptors->CaIds());
-#endif /* TTXTSUBS */
            Channel->SetSubtitlingDescriptors(SubtitlingTypes, CompositionPageIds, AncillaryPageIds);
            }
         Channel->SetCaDescriptors(CaDescriptorHandler.AddCaDescriptors(CaDescriptors));
