@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.c 2.4 2010/01/16 11:18:53 kls Exp $
+ * $Id: timers.c 2.5 2011/08/06 13:13:54 kls Exp $
  */
 
 #include "timers.h"
@@ -29,6 +29,7 @@ cTimer::cTimer(bool Instant, bool Pause, cChannel *Channel)
 {
   startTime = stopTime = 0;
   lastSetEvent = 0;
+  deferred = 0;
   recording = pending = inVpsMargin = false;
   flags = tfNone;
   if (Instant)
@@ -46,9 +47,6 @@ cTimer::cTimer(bool Instant, bool Pause, cChannel *Channel)
      stop -= 2400;
   priority = Pause ? Setup.PausePriority : Setup.DefaultPriority;
   lifetime = Pause ? Setup.PauseLifetime : Setup.DefaultLifetime;
-#ifdef USE_PINPLUGIN
-  fskProtection = 0;
-#endif /* PINPLUGIN */
   *file = 0;
   aux = NULL;
   event = NULL;
@@ -65,6 +63,7 @@ cTimer::cTimer(const cEvent *Event)
 {
   startTime = stopTime = 0;
   lastSetEvent = 0;
+  deferred = 0;
   recording = pending = inVpsMargin = false;
   flags = tfActive;
   if (Event->Vps() && Setup.UseVps)
@@ -87,9 +86,6 @@ cTimer::cTimer(const cEvent *Event)
      stop -= 2400;
   priority = Setup.DefaultPriority;
   lifetime = Setup.DefaultLifetime;
-#ifdef USE_PINPLUGIN
-  fskProtection = 0;
-#endif /* PINPLUGIN */
   *file = 0;
   const char *Title = Event->Title();
   if (!isempty(Title))
@@ -101,9 +97,6 @@ cTimer::cTimer(const cEvent *Event)
      }
   aux = NULL;
   event = NULL; // let SetEvent() be called to get a log message
-#ifdef USE_PINPLUGIN
-  cStatus::MsgTimerCreation(this, Event);
-#endif /* PINPLUGIN */
 }
 
 cTimer::cTimer(const cTimer &Timer)
@@ -127,6 +120,7 @@ cTimer& cTimer::operator= (const cTimer &Timer)
      startTime    = Timer.startTime;
      stopTime     = Timer.stopTime;
      lastSetEvent = 0;
+     deferred = 0;
      recording    = Timer.recording;
      pending      = Timer.pending;
      inVpsMargin  = Timer.inVpsMargin;
@@ -138,9 +132,6 @@ cTimer& cTimer::operator= (const cTimer &Timer)
      stop         = Timer.stop;
      priority     = Timer.priority;
      lifetime     = Timer.lifetime;
-#ifdef USE_PINPLUGIN
-     fskProtection = Timer.fskProtection;
-#endif /* PINPLUGIN */
      strncpy(file, Timer.file, sizeof(file));
      free(aux);
      aux = Timer.aux ? strdup(Timer.aux) : NULL;
@@ -335,9 +326,6 @@ bool cTimer::Parse(const char *s)
         result = false;
         }
      }
-#ifdef USE_PINPLUGIN
-  fskProtection = aux && strstr(aux, "<pin-plugin><protected>yes</protected></pin-plugin>");
-#endif /* PINPLUGIN */
   free(channelbuffer);
   free(daybuffer);
   free(filebuffer);
@@ -437,6 +425,10 @@ bool cTimer::Matches(time_t t, bool Directly, int Margin) const
         day = 0;
      }
 
+  if (t < deferred)
+     return false;
+  deferred = 0;
+
   if (HasFlags(tfActive)) {
      if (HasFlags(tfVps) && event && event->Vps()) {
         if (Margin || !Directly) {
@@ -604,6 +596,12 @@ void cTimer::SetPriority(int Priority)
   priority = Priority;
 }
 
+void cTimer::SetDeferred(int Seconds)
+{
+  deferred = time(NULL) + Seconds;
+  isyslog("timer %s deferred for %d seconds", *ToDescr(), Seconds);
+}
+
 void cTimer::SetFlags(uint Flags)
 {
   flags |= Flags;
@@ -647,37 +645,6 @@ void cTimer::OnOff(void)
   Matches(); // refresh start and end time
 }
 
-#ifdef USE_PINPLUGIN
-void cTimer::SetFskProtection(int aFlag)
-{
-   char* p;
-   char* tmp = 0;
-
-   fskProtection = aFlag;
-
-   if (fskProtection && (!aux || !strstr(aux, "<pin-plugin><protected>yes</protected></pin-plugin>")))
-   {
-      // add protection info to aux
-
-      if (aux) { tmp = strdup(aux); free(aux); }
-      if (asprintf(&aux,"%s<pin-plugin><protected>yes</protected></pin-plugin>", tmp ? tmp : "") < 0 )
-         aux = NULL;
-   }
-   else if (!fskProtection && aux && (p = strstr(aux, "<pin-plugin><protected>yes</protected></pin-plugin>")))
-   {
-      // remove protection info to aux
-
-      if (asprintf(&tmp, "%.*s%s", p-aux, aux, p+strlen("<pin-plugin><protected>yes</protected></pin-plugin>")) >= 0 ) {
-         free(aux);
-         aux = strdup(tmp);
-         }
-   }
-
-   if (tmp)
-      free(tmp);
-}
-#endif /* PINPLUGIN */
-
 // --- cTimers ---------------------------------------------------------------
 
 cTimers Timers;
