@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: timers.h 2.0 2008/02/16 14:33:23 kls Exp $
+ * $Id: timers.h 2.1 2011/08/06 12:59:32 kls Exp $
  */
 
 #ifndef __TIMERS_H
@@ -29,6 +29,7 @@ class cTimer : public cListObject {
 private:
   mutable time_t startTime, stopTime;
   time_t lastSetEvent;
+  mutable time_t deferred; ///< Matches(time_t, ...) will return false if the current time is before this value
   bool recording, pending, inVpsMargin;
   uint flags;
   cChannel *channel;
@@ -37,9 +38,6 @@ private:
   int start;
   int stop;
   int priority;
-#ifdef USE_PINPLUGIN
-  int fskProtection;
-#endif /* PINPLUGIN */
   int lifetime;
   mutable char file[MaxFileName];
   char *aux;
@@ -61,13 +59,11 @@ public:
   int Start(void) const { return start; }
   int Stop(void) const { return stop; }
   int Priority(void) const { return priority; }
-#ifdef USE_PINPLUGIN
-  int FskProtection(void) const { return fskProtection; }
-#endif /* PINPLUGIN */
   int Lifetime(void) const { return lifetime; }
   const char *File(void) const { return file; }
   time_t FirstDay(void) const { return weekdays ? day : 0; }
   const char *Aux(void) const { return aux; }
+  time_t Deferred(void) const { return deferred; }
   cString ToText(bool UseChannelID = false) const;
   cString ToDescr(void) const;
   const cEvent *Event(void) const { return event; }
@@ -91,10 +87,8 @@ public:
   void SetPending(bool Pending);
   void SetInVpsMargin(bool InVpsMargin);
   void SetPriority(int Priority);
+  void SetDeferred(int Seconds);
   void SetFlags(uint Flags);
-#ifdef USE_PINPLUGIN
-  void SetFskProtection(int aFlag);
-#endif /* PINPLUGIN */
   void ClrFlags(uint Flags);
   void InvFlags(uint Flags);
   bool HasFlags(uint Flags) const;
